//+------------------------------------------------------------------+
//|                                       multi_symbol_timeframe.mqh |
//|                                      Copyright 2017, nicholishen |
//|                         https://www.forexfactory.com/nicholishen |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, nicholishen"
#property link      "https://www.forexfactory.com/nicholishen"
#property strict
#include <stdlib.mqh>
#include <Arrays\ArrayString.mqh>

/*
500 ms time-out. Increase this time if functions are failing to perform 
and/or netowork bandwidth is low.            
*/
#define TIMEOUT 1200 
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#define __MACRO_BEGIN__                                     \
   const static uint time_out=TIMEOUT;                      \
   uint              ms          = GetTickCount();          \
   uint              milliseconds= 0;                       \
   int               error;                                 \
   do{                                                      \
      ResetLastError();                                     \
      milliseconds=GetTickCount()-ms;
//+------------------------------------------------------------------+
#define __MACRO_END__                                       \
      error=GetLastError();                                 \
   }                                                        \
   while(error!=ERR_NO_ERROR && milliseconds<=time_out && !IsStopped());    \
   return error==ERR_NO_ERROR;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iBid(double &result,const string symbol)
  {
   SymbolSelect(symbol,true);
   __MACRO_BEGIN__
   result=SymbolInfoDouble(symbol,SYMBOL_BID);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double iBid(const string symbol)
  {
   double bid;
   if(!iBid(bid,symbol))
      return 0.0;
   return bid;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iAsk(double &result,const string symbol)
  {
   SymbolSelect(symbol,true);
   __MACRO_BEGIN__
   result=SymbolInfoDouble(symbol,SYMBOL_ASK);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double iAsk(const string symbol)
  {
   double ask;
   if(!iAsk(ask,symbol))
      return 0.0;
   return ask;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool InitSymbolsHistory(const CArrayString      &symbols,
                        ENUM_TIMEFRAMES   tf_start = PERIOD_M1,
                        ENUM_TIMEFRAMES   tf_end   = PERIOD_MN1,
                        bool              verbose  = false)
{
   string arr[];
   int total = ArrayResize(arr,symbols.Total());
   for(int i=0;i<total;i++)
      arr[i]=symbols[i];
   return InitSymbolsHistory(arr,tf_start,tf_end,verbose);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool InitSymbolsHistory(const string      &symbols[],
                        ENUM_TIMEFRAMES   tf_start = PERIOD_M1,
                        ENUM_TIMEFRAMES   tf_end   = PERIOD_MN1,
                        bool              verbose  = false)
  {
   const static ENUM_TIMEFRAMES tf[]=
     {
      PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,
      PERIOD_H1,PERIOD_H4,PERIOD_D1,PERIOD_W1,PERIOD_MN1
     };
   bool res = true;
   for(int i=0;i<ArraySize(symbols);i++)
     {
      SymbolSelect(symbols[i],true);
      for(int j=0;j<ArraySize(tf);j++)
        {
         if(tf[j]<tf_start)
            continue;
         if(tf[j]>tf_end)
            break;
         if(!IsDataReady(symbols[i],tf[j]))
         {
            if(verbose)
               Print(__FUNCTION__," ",ErrorDescription(GetLastError()));
            res=false;
         }
            
         if(verbose)
            Print(symbols[i]," "+EnumToString(tf[j])+" ",string(res));
        }
     }
   return res;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsDataReady(const string symbol,const int timeframe)
  {
   SymbolSelect(symbol,true);
   int result;
   return iBars(result,symbol,timeframe);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iOpen(double &result,const string symbol,int timeframe,int shift)
  {
   __MACRO_BEGIN__
   result=iOpen(symbol,timeframe,shift);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iHigh(double &result,const string symbol,int timeframe,int shift)
  {
   __MACRO_BEGIN__
   result=iHigh(symbol,timeframe,shift);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iLow(double &result,const string symbol,int timeframe,int shift)
  {
   __MACRO_BEGIN__
   result=iLow(symbol,timeframe,shift);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iClose(double &result,const string symbol,int timeframe,int shift)
  {
   __MACRO_BEGIN__
   result=iClose(symbol,timeframe,shift);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iVolume(long &result,const string symbol,int timeframe,int shift)
  {
   __MACRO_BEGIN__
   result=iVolume(symbol,timeframe,shift);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iTime(datetime &result,const string symbol,int timeframe,int shift)
  {
   __MACRO_BEGIN__
   result=iTime(symbol,timeframe,shift);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iBarShift(int &result,const string symbol,int timeframe,datetime time,bool exact=false)
  {
   __MACRO_BEGIN__
   result=iBarShift(symbol,timeframe,time,exact);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iBars(int &result,const string symbol,int timeframe)
  {
   __MACRO_BEGIN__
   result=iBars(symbol,timeframe);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iHighest(int &result,const string symbol,int timeframe,ENUM_SERIESMODE type,int count=WHOLE_ARRAY,int start=0)
  {
   __MACRO_BEGIN__
   result=iHighest(symbol,timeframe,type,count,start);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool iLowest(int &result,const string symbol,int timeframe,ENUM_SERIESMODE type,int count=WHOLE_ARRAY,int start=0)
  {
   __MACRO_BEGIN__
   result=iLowest(symbol,timeframe,type,count,start);
   __MACRO_END__
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+